{-----------------------------------------------------------------------------
 Unit Name: uApptWhoIterator
 Author:    rob
 Purpose:   Iterator for a list of appointments, filtered by Who property.
 History:
-----------------------------------------------------------------------------}


unit uApptWhoIterator;

interface

uses
  uIApptIterator
  , uAppointments
  , uAppointment
  , sysutils
  ;

type

  TApptWhoIterator = class(TInterfacedObject, IApptIterator)
  private

    FApptList: TAppointments;               // The list we're iterating over
    FApptNo: integer;                       // Pointer to the current appointment
    FNext: integer;                         // Pointer to the next appointment
    FPrev: integer;                         // Pointer to the prev appointment
    FWho: string;                           // The string to match

    function First: TAppointment;
    function Last: TAppointment;
    function HasNext: boolean;
    function HasPrev: boolean;
    function Current: TAppointment;
    function Next: TAppointment;
    function Prev: TAppointment;

  public

    constructor Create(ApptList: TAppointments; Who: string);

    property Who: string
      read FWho;

  end;

implementation

uses
  uApptIterator
  ;

{ TApptWhoIterator }

constructor TApptWhoIterator.Create(ApptList: TAppointments; Who: string);
begin
  Assert(Assigned(ApptList), 'ApptList must be assigned');
  inherited Create;
  FApptList := ApptList;
  FWho := Who;
  First;
end;

function TApptWhoIterator.Current: TAppointment;
begin
  // If we're pointing at an appointment, return it, otherwise return nil
  if (FApptNo < FApptList.Count) and (FApptNo >= 0) then
    Result := FApptList.Items[FApptNo]
  else
    Result := nil;
end;

function TApptWhoIterator.First: TAppointment;
begin
  // Go to the first appointment that matches the filter criteria & return it
  FApptNo := 0;                               // Test the first appointment in the list
  while (FApptNo < FApptList.Count) and
        (FApptList.Items[FApptNo].Who <> FWho) do
    Inc(FApptNo);
  Result := Current;
end;

function TApptWhoIterator.HasNext: boolean;
begin
  // Look for a later appointment that matches the filter criteria. Set FNext to
  // point to it. Return false if we don't find one
  FNext := Succ(FApptNo);
  while (FNext < FApptList.Count) and
        ((FApptList.Items[FNext].Who <> FWho)) do
    Inc(FNext);
  Result := FNext < FApptList.Count;
end;

function TApptWhoIterator.HasPrev: boolean;
begin
  // Look for a prior appointment that matches the filter criteria. Set FNext to
  // point to it. Return false if we don't find one
  FPrev := Pred(FApptNo);
  while (FPrev >= 0) and
        ((FApptList.Items[FPrev].Who <> FWho)) do
    Dec(FPrev);
  Result := FPrev >= 0;
end;

function TApptWhoIterator.Last: TAppointment;
begin
  // Go to the last appointment that matches the filter criteria & return it
  FApptNo := Pred(FApptList.Count);           // Test the last appointment in the list
  while (FApptNo >= 0) and
        (FApptList.Items[FApptNo].Who <> FWho) do
    Dec(FApptNo);
  Result := Current;
end;

function TApptWhoIterator.Next: TAppointment;
begin
  // Call HasNext to find the next appointment. If it finds one, return it &
  // move the list pointer, otherwise raise an exception.
  if HasNext then
  begin
    Result := Current;
    FApptNo := FNext;
  end
  else
  begin
    raise EApptIteratorException.Create('No more appointments');
  end;
end;

function TApptWhoIterator.Prev: TAppointment;
begin
  // Call HasPrev to find the previous appointment. If it finds one, return it &
  // move the list pointer, otherwise raise an exception.
  if HasPrev then
  begin
    Result := Current;
    FApptNo := FPrev;
  end
  else
  begin
    raise EApptIteratorException.Create('No more appointments');
  end;
end;

end.
